<?php
// $Id$
/**
 * @file
 * Handles all of the communication with [us/eu].battle.net.
 * 
 */

/**
 * Menu calls this function to update a toon.
 */
function wowtoon_force_update($toon) {
  $results = wowtoon_update_armory($toon->tid);
  if ($results->tid) {
    wowtoon_update_feed($toon);
    module_invoke_all('toon_updated', $toon);
  }
  else {
    drupal_set_message(t('Error Loading Toon %toon(@url)', array('%toon' => $toon->wowtoon_name[LANGUAGE_NONE][0]['value'], '@url' => $toon->url)));
  }
  
  drupal_goto('toon/' . $toon->tid);
}
/**
 * Menu calls this function to update a guild.
 */
function wowguild_force_update($guild) {
  $results = wowguild_update_armory($guild->gid);
  
  if (!empty($results->gid)) {
    wowguild_update_feed($guild);
    wowguild_update_progression($guild);
  }
  else {
    drupal_set_message(t('Error Loading Guild %guild(@url)', array('%guild' => $guild->wowtoon_name[LANGUAGE_NONE][0]['value'], '@url' => $guild->url)));
  }
  drupal_goto('guild/' . $guild->gid);
}

/**
 * Updates the toon from the armory, or checks for existance of toon.
 * 
 * @param string $realm
 *   - This can be the realm name, OR the {wowtoon}.tid to be updated.
 * @param string $name
 * @param integer $validate_slot
 *   - If this is passed see if the user has equiped / unequiped the item in the slot and return TRUE/FALSE. 
 * 
 * If {wowtoon}.tid is passed, then we know it exists, so just update the toon.
 * 
 * If $realm / $name combo is passed, we are checking for existance (and possibly doing a first load) of a toon.
 *   If toon exists and is available, the resultant {wowtoon} will be returned.  Otherwise one of the above error constants will be returned.
 * 
 */
function wowtoon_update_armory($realm = '', $name = '', $validate_slot = NULL) {
  global $user;
  
  $toon = new stdClass();
  if (is_numeric($realm)) {
    // We are updating an existing toon.
    $toon = wowtoon_load($realm);
    if ($toon === FALSE) {
      watchdog('error', 'Recevied an update for {wowtoon}.tid = @tid, but it does not exist.', array('@tid' => $realm));
      return FALSE;
    }
  }
  else {
    // Run Error Checks
    $toon->toon_key = wowtoon_getkey($realm, $name);
    if (_wowtoon_validate_realm($realm) === FALSE) {
      return WOWTOON_UPDATE_INVALID_REALM;
    }
    if (trim($name) == '') {
      return WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
    }
    
    // See if toon exists and is owned.
    $result = db_query('SELECT tid FROM {wowtoon_entity} WHERE toon_key = :toon_key', array(':toon_key' => $toon->toon_key))->fetchObject();
    if ($result) {
      $toon = wowtoon_load($result->tid);
      if ($validate_slot == NULL) {
        return $toon;
      }
    }
    else {
      $toon->realm = $realm;
      $toon->name = $name;
    }
  }
  
  $zone = substr($toon->realm, strlen($toon->realm) - 2);
  $realm = substr($toon->realm, 0, strlen($toon->realm) - 3);
  
  // wowhead uses www, the armory uses en
  $language = variable_get('wowtoon_default_language', 'www');
  if ($language == 'www') {
    $language = 'en';
  }
  // Verify that this zone is available to the selected language.
  if (strtolower($zone) == 'us' && ($language == 'fr' || $language == 'de' || $language == 'ru')) {
    $language = 'en';
  }
  
  
  $headers = array();
  $headers["User-Agent"] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/20070219 Firefox/2.0.0.2";
  // Battle.Net does not seem to respond to this header.
  if (!empty($toon->updated)) {
    $headers['If-Modified-Since'] = gmdate('D, d M Y H:i:s', $toon->updated) . ' GMT';
  }
  // Request feed.
  $url = sprintf('http://%s.battle.net/wow/%s/character/%s/%s/', strtolower($zone), $language, urlencode(str_replace(array("'", ' '), array("", '-'), $realm)), str_replace(' ', '%20', $toon->name));
  $toon->url = $url;
  $url .= 'advanced';
  
  $toon->update_attempted = REQUEST_TIME;
  
  if (module_exists('qpcache')) {
    $cache = qpcache_get($url);
    if (empty($cache)) {
      $result = drupal_http_request($url, $headers);
    }
    else {
      //drupal_set_message("Cache Hit");
      $result->data = $cache->xml;
    }
  }
  else {
    $result = drupal_http_request($url, $headers);
  }
  
  $error_num = 0;
  
  try {
    $qp = qp($result->data, NULL, array('use_parser' => 'xml'));
    $error = $qp->find('#server-error')->innerHTML();
    if ($error) {
      if (strpos($error, 'Only characters level 10 or higher can be viewed')) {
        $error_num = WOWTOON_UPDATE_CHARACTER_TOO_LOW;
      }
      elseif (strpos($error, 'Character Not Available')) {
        $error_num = WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
      }
      else {
        $error_num = WOWTOON_UPDATE_INVALID_PAGE;
      }
    }
    else {
      $error_num = WOWTOON_UPDATE_CHARACTER_AVAILABLE;
    }
  } catch (Exception $e) {
    $error_num = WOWTOON_UPDATE_INVALID_PAGE;
  }
  
  $qp->top(); $toon->name = trim(strip_tags($qp->find('.name')->innerHTML()));
  // Idiot Check, this should never happen.
  if (empty($toon->name)) {
    $error_num = WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
  }
  // See if we hit an error getting data for this toon.
  if ($error_num != WOWTOON_UPDATE_CHARACTER_AVAILABLE) {
    if (!empty($toon->tid)) {
      db_update('wowtoon_entity')
        ->fields(array(
          'update_status' => $error_num,
          'update_attempted' => $toon->update_attempted,
          'url' => $toon->url,
        ))
        ->condition('tid', $toon->tid)
        ->execute();
    }
    return $error_num;
  }
  else {
    $toon->update_status = WOWTOON_UPDATE_CHARACTER_AVAILABLE;
  }
  
  // Cache data if needed.
  if (empty($cache) && module_exists('qpcache')) {
    qpcache_set($url, $result->data, '+' . variable_get('wowtoon_updatedelta', 12) . ' hours'); 
  }
  
  // Are we attempting to validate?
  if ($validate_slot) {
    $qp->top(); $qp->find('#summary-inventory')->children();
    foreach ($qp as $ele) {
      $itemslot = $ele->attr('data-type');
      $key = $ele->attr('data-id');
      
      if (abs($validate_slot) == $key) {
        $ele->firstChild()->firstChild()->firstChild();
        if ($validate_slot > 0) {
          // Check to see if we added something.
          return $ele->hasClass('item');
        }
        else {
          // Check to see if we removed something.
          return $ele->hasClass('empty');
        }
      }
    }
    // This shouldn't happen, the item slot did not exist.
    return FALSE;
  }
  

  $qp->top(); $title = $qp->find('.title')->innerHTML();
  
  // Title may contain a bogus character.  Make sure title is more than two characters long.
  if (trim($title) && strlen(trim($title)) > 2) {
    if (substr($title, 0, 1) == ' ') {
      $toon->fullname = $toon->name . $title;
    }
    else {
      $toon->fullname = $title . $toon->name;
    }
  }
  else {
    $toon->fullname = $toon->name;
  }
  
  $qp->top(); $toon->battlegroup = $qp->find('#profile-info-realm')->attr('data-battlegroup');
  $qp->top(); $toon->guild_name = $qp->find('.guild')->firstChild()->innerHTML();
  
  // Check Scripts
  // Grab text within scripts and massage it so that it'll return a correct array from json_decode.
  // Enclose left hand arguments in double quotes and unescape \' to '.  Enclose entire string in [ ]'s.
  $qp->top(); $qp->find('script');
  $i=0;
  $allstats = array();
  foreach ($qp as $script) {
    if ($script->innerHTML()) {
      $txt = $script->innerHTML();
      if (strpos($txt, 'Summary.Stats') !== FALSE) {
        $allstats = _wowtoon_parse_json($txt, 'Summary.Stats(', '});');
      } /*elseif (strpos($txt, 'Summary.Inventory(') !== FALSE) {
        $toon->progression = _wowtoon_parse_json($txt, 'Summary.Inventory(', '});');
        }
      }*/ 
      elseif (strpos($txt, 'Summary.RaidProgression(') !== FALSE) {
        $toon->progression = _wowtoon_parse_json($txt, 'Summary.RaidProgression(', '});');
      }
      elseif (strpos($txt, 'Summary.Audit(') !== FALSE) {
        $toon->audit = _wowtoon_parse_json($txt, 'Summary.Audit(', '});');
      }
      elseif (strpos($txt, 'var MsgSummary =') !== FALSE) {
        $msg = _wowtoon_parse_json($txt, 'var MsgSummary =', '};');
        variable_set('wowtoon_messagesummary', $msg);
      }
    }
  }
  
  $qp->top(); $toon->classId = str_replace('under-name color-c', '', $qp->find('.under-name')->attr('class'));
  $qp->top(); $toon->class = $qp->find('.class')->innerHTML();
  $qp->top(); $toon->level = trim(strip_tags($qp->find('.level')->innerHTML()));
  $qp->top(); $toon->race = $qp->find('.race')->innerHTML();
  // DKs do not show RaceId.  Cheat and lets look it up.
  switch ($toon->race) {
    case 'Human': $toon->raceId = 1;      $toon->factionId = WOWTOON_ALLIANCE_ID; $toon->faction = 'Alliance'; break;
    case 'Orc': $toon->raceId = 2;        $toon->factionId = WOWTOON_HORDE_ID;    $toon->faction = 'Horde'; break;
    case 'Dwarf': $toon->raceId = 3;      $toon->factionId = WOWTOON_ALLIANCE_ID; $toon->faction = 'Alliance'; break;
    case 'Night Elf': $toon->raceId = 4;  $toon->factionId = WOWTOON_ALLIANCE_ID; $toon->faction = 'Alliance'; break;
    case 'Undead': $toon->raceId = 5;     $toon->factionId = WOWTOON_HORDE_ID;    $toon->faction = 'Horde'; break;
    case 'Tauren': $toon->raceId = 6;     $toon->factionId = WOWTOON_HORDE_ID;    $toon->faction = 'Horde'; break;
    case 'Gnome': $toon->raceId = 7;      $toon->factionId = WOWTOON_ALLIANCE_ID; $toon->faction = 'Alliance'; break;
    case 'Troll': $toon->raceId = 8;      $toon->factionId = WOWTOON_HORDE_ID;    $toon->faction = 'Horde'; break;
    case 'Goblin': $toon->raceId = 9;     $toon->factionId = WOWTOON_HORDE_ID;    $toon->faction = 'Horde'; break;
    case 'Blood Elf': $toon->raceId = 10; $toon->factionId = WOWTOON_HORDE_ID;    $toon->faction = 'Horde'; break;
    case 'Draenei': $toon->raceId = 11;   $toon->factionId = WOWTOON_ALLIANCE_ID; $toon->faction = 'Alliance'; break;
    case 'Worgen': $toon->raceId = 22;    $toon->factionId = WOWTOON_ALLIANCE_ID; $toon->faction = 'Alliance'; break;
  }
  
  
  if ($toon->guild_name) {
    // See if Guild Exists in Database
    $guild = wowguild_load_by_realm_name($toon->realm, $toon->guild_name);
    if (empty($guild->gid)) {
      $guild->name = $toon->guild_name;
      $guild->realm = $toon->realm;
      $guild->faction = $toon->faction;
      $guild->factionId = $toon->factionId;
      $guild->added_to_database = REQUEST_TIME;
      $guild->url = sprintf('http://%s.battle.net/wow/en/guild/%s/%s/', strtolower($zone), urlencode(str_replace(array("'", ' '), array("", '-'), $realm)), urlencode(($guild->name)));
      wowguild_save($guild);
    }
    $toon->gid = $guild->gid; 
  }
  
  
  
  $qp->top(); $toon->achievement_points = trim(strip_tags($qp->find('.achievements')->innerHTML()));
  
  $qp->top(); $toon->avg_ilevel = trim(strip_tags($qp->find('.summary-averageilvl')->find('.equipped')->innerHTML()));
  $qp->top(); $toon->avg_ilevel_best = trim(strip_tags($qp->find('#summary-averageilvl-best')->innerHTML()));
  
  // Grab the background images
  $qp->top(); $style = $qp->find(':root style')->html();
  $a = strpos($style, 'background: url("') + strlen('background: url("');
  $b = strpos($style, '")', $a);
  $toon->race_background = substr($style, $a, $b - $a);
  
  $a = strpos($style, 'background-image: url("') + strlen('background-image: url("');
  $b = strpos($style, '")', $a);
  $full_profile_image = substr($style, $a, $b - $a);
  $a = strpos($style, 'background-image: url("') + strlen('background-image: url("');
  $b = strpos($style, '-profilemain', $a);
  $toon->armory_render = substr($style, $a, $b - $a);
  
  // Grab the 7-0.jpg part of the path.  The 0 is the gender.
  $b = strpos($style, '");', $a);
  $c = substr($style, $a, $b - $a);
  $a = strrpos($c, '/') + 1;
  $b = strrpos($c, '.');
  list($null, $toon->genderId) = explode('-', substr($c, $a, $b - $a));
  $toon->gender = $toon->genderId == '1'?'Female':'Male';
  
  $qp->top(); $toon->lifetimehonorablekills = trim(strip_tags($qp->find('.kills')->find('.value')->innerHTML()));
  
  $qp->top(); $qp->find('.profession-details');
  $toon->professions = array();
  foreach ($qp as $key => $ele) {
    $toon->professions[$key] = array();
    $toon->professions[$key]['name'] = $ele->find('.name')->innerHTML();
    $toon->professions[$key]['value'] = $ele->next()->innerHTML();
    $toon->professions[$key]['max'] = WOWTOON_MAXIMUM_PROFESSION_LEVEL;
  }
  
  // Loop over talents
  $qp->top(); $qp->find('#summary-talents')->firstChild()->children();
  $activeset = -999; 
  $specs = array();
  foreach ($qp as $key => $ele) {
    $specs[$key] = array();
    $specs[$key]['active'] = $ele->firstChild()->hasClass('active')?'1':'0';
    // Remove prefix: http://us.battle.net/wow-assets/static/images/icons/36/spell_shadow_deathcoil.jpg
    $icon = $ele->find('.icon')->find('img')->attr('src');
    $specs[$key]['icon'] = array_pop(explode('/', $icon));
    $specs[$key]['name'] = $ele->parent()->parent()->find('.name')->innerHTML();
    $specs[$key]['build'] = trim(strip_tags($ele->next()->innerHTML()));
  }
  
  // Loop over items
  $items = array();
  $qp->top(); $qp->find('#summary-inventory')->children();
  foreach ($qp as $ele) {
    $itemslot = $ele->attr('data-type');
    $key = $ele->attr('data-id');
    $items[$key] = array();
    //$items[$key]['quality'] = str_replace('name color-q', '', $detail->attr('class'));
    $items[$key]['slot'] = $ele->attr('data-type');
    $items[$key]['data-id'] = $ele->attr('data-id');
    
    $icon = $ele->find('img')->attr('src');
    $items[$key]['icon'] = array_pop(explode('/', $icon));
    $ele->parent();
    
    // Grab Item Number (blizzard change, nolonger in attr('data-item'))
    $url = $ele->attr('href');
    $items[$key]['i'] = array_pop(explode('/', $url));
    
    $items[$key]['data-item'] = $ele->attr('data-item');
    if ($items[$key]['data-item']) {
      $attrs = explode('&', $items[$key]['data-item']);
      foreach ($attrs as $attr) {
        list($a, $b) = explode('=', $attr);
        $items[$key][$a] = $b;
      }
    }
    $ele->parent();
    
    // Loop Over Details
    $ele->find('.details')->children();
    foreach ($ele as $detail) {
      if ($detail->hasClass('name-shadow')) {
        $items[$key]['name'] = $detail->innerHTML();
      }
      if ($detail->hasClass('name')) {
        $items[$key]['quality'] = str_replace('name color-q', '', $detail->attr('class'));
      }
      if ($detail->hasClass('enchant-shadow')) {
        $items[$key]['enchant-name'] = $detail->innerHTML();
      }
      if ($detail->hasClass('level')) {
        $items[$key]['level'] = $detail->innerHTML();
      }
    }
    $ele->find('.icon-socket');
    $i=0;
    foreach ($ele as $gem) {
      $items[$key]['sockets'][$i]['socket'] = array_pop(explode('-', $gem->attr('class')));
      $items[$key]['sockets'][$i]['id'] = array_pop(explode('/', $gem->firstChild()->attr('href')));
      $items[$key]['sockets'][$i]['icon'] = array_pop(explode('/', $gem->firstChild()->attr('src')));
      $i++;
    }
  }
  
  
  $qp->top(); $updated = trim($qp->find('.summary-lastupdate')->innerHTML());
  $toon->updated_on_armory = strtotime(substr($updated, strlen($updated) - 10));
  
  
  $stats = array();
  $qp->top(); $qp->find('.summary-stats-column');
  foreach ($qp as $ele) {
    $style = $ele->attr('style');
    if ($style != 'display: none') {
      $key = $ele->firstChild()->innerHTML();
      $stats[$key] = array();
      $ele->next()->children();
      $count = $ele->size();
      
      $i=0;
      foreach ($ele as $stat) {
        $stats[$key][$stat->attr('data-id')] = $stat->find('.value')->innerHTML(); 
      }
    }
  }
  
  // From here down, we are dealing with additional databases, so we need a tid.
  if (!isset($toon->tid)) {
    $toon->added_to_database = REQUEST_TIME;
    wowtoon_save($toon);
    // Load achievements for 1st time.
    wowtoon_update_feed($toon);
  }
  
  // Load current sets
  $results = db_query('SELECT * FROM {wowtoon_sets} WHERE tid = :tid', array(':tid' => $toon->tid));
  $sets = array();
  foreach ($results as $set) {
    $sets[$set->set_id] = $set;
  }
  
  // Check Spec and Sets
  for ($i=0;$i<count($specs);$i++) {
    $sets[$i]->tid = $toon->tid;
    $sets[$i]->set_id = $i;
    $sets[$i]->icon = $specs[$i]['icon'];
    $sets[$i]->spec = sprintf("%s (%s)", $specs[$i]['name'], $specs[$i]['build']);
    
    if ($specs[$i]['active'] == '1') {
      $sets[$i]->active = 1;
      $sets[$i]->stats = $stats;
      $sets[$i]->allstats = $allstats;
      $sets[$i]->items = $items;

      // See if we need to download background image
      if (variable_get('wowtoon_cache_backgrounds', 0)) {
        $real_background_directory = variable_get('wowtoon_cache_backgrounds_path', 'public://wowtoon_background_cache');
        if (substr($real_background_directory, 0, 9) == 'public://') {
          $url_background_directory = file_create_url($real_background_directory);
          $real_background_directory = drupal_realpath($real_background_directory);
        }
        else {
          $url_background_directory = variable_get('wowtoon_cache_backgrounds_url', file_create_url('public://wowtoon_background_cache'));
        }
        
        $real_filename = sprintf("%s/toon-%s-%d.jpg", $real_background_directory, urlencode($toon->toon_key), $i);
        $url_filename = sprintf("%s/toon-%s-%d.jpg", $url_background_directory, urlencode($toon->toon_key), $i);
        if (!file_exists($real_background_directory)) {
          drupal_mkdir($real_background_directory, NULL, TRUE);
        }
        if (copy($full_profile_image, $real_filename)) {
          $sets[$i]->background = $url_filename;
        }
        else {
          watchdog('error', 'Could not copy background image %from to %to directory', array('%from' => $full_profile_image, '%to' => $real_filename));
          $sets[$i]->background = $full_profile_image;
        }
      }
    }
    else {
      // Unset these so we don't overwrite old values.
      unset($sets[$i]->stats);
      unset($sets[$i]->allstats);
      unset($sets[$i]->items);
      $sets[$i]->active = 0;
    }
    if (isset($set->id)) {
      drupal_write_record('wowtoon_sets', $sets[$i], 'id');
    }
    else {
      drupal_write_record('wowtoon_sets', $sets[$i]);
    }
  }
  $toon->updated = REQUEST_TIME;
  
  wowtoon_save($toon);
  return $toon;
}


/**
 * Updates updates the character feed of a toon.
 * 
 * @param {wowtoon_entity} $toon
 *   - This must be a valid toon.
 *   
 * TODO: Use JSON request: http://blizzard.github.com/api-wow-docs/#id3682393
 * 
 */
function wowtoon_update_feed($toon) {
  // Generate conditional GET headers.
  $headers = array();
  $headers["User-Agent"] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/20070219 Firefox/2.0.0.2";
//  $headers['If-Modified-Since'] = gmdate('D, d M Y H:i:s', $feed['modified']) .' GMT';

  $name = $toon->name;
  $realm = $toon->realm;
  $zone = substr($toon->realm, strlen($toon->realm) - 2);
  
  // wowhead uses www, the armory uses en
  $language = variable_get('wowtoon_default_language', 'en');
  if ($language == 'www') {
    $language = 'en';
  }
  // Verify that this zone is available to the selected language.
  if (strtolower($zone) == 'us' && ($language == 'fr' || $language == 'de' || $language == 'ru')) {
    $language = 'en';
  }
  

  $url = $toon->url . 'feed';

  if (module_exists('qpcache')) {
    $cache = qpcache_get($url);
    if (empty($cache)) {
      $result = drupal_http_request($url, $headers);
    }
    else {
      //drupal_set_message("Cache Hit");
      $result->data = $cache->xml;
    }
  }
  else {
    $result = drupal_http_request($url, $headers);
  }
  
  // Cache data if needed.
  if (empty($cache) && module_exists('qpcache')) {
    qpcache_set($url, $result->data, '+' . variable_get('wowtoon_updatedelta', 12) . ' hours'); 
  }
  
  try {
    $qp = qp($result->data, NULL, array('use_parser' => 'xml'));
    $error = $qp->find('#server-error')->innerHTML();
    if ($error) {
      if (strpos($error, 'Character Not Available')) {
        return WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
      }
      else {
        return WOWTOON_UPDATE_INVALID_PAGE;
      }
    }
  } catch (Exception $e) {
    return WOWTOON_UPDATE_INVALID_PAGE;
  }
  
  
  $toon_key = wowtoon_getkey($realm, $name);
 
  $ach = array();
  
  $snapshot_time = $toon->updated_on_armory;
  $qp->top()->find('.activity-feed')->children();
  foreach ($qp as $item) {
    unset($newobj);
    $newobj->keyid = $toon->tid;
    $class = $item->attr('class');
    $newobj->class = $class;
    if (strpos($class, 'ach') !== FALSE) {
      $newobj->type = WOWTOON_FEED_ACHIEVEMENT;
      $txt = $item->firstChild()->firstChild()->firstChild()->firstChild()->attr('style');
      $a = strpos($txt, 'url("') + 5;
      $b = strpos($txt, '"', $a);
      $newobj->icon = substr($txt, $a, $b - $a);
      $txt = $item->parent()->parent()->innerHTML();
      $a = strpos($txt, '</a>') + 5;
      $txt = trim(substr($txt, $a));
      $txt = preg_replace(array("/achievement#.*:a/", '/onmouseover=\".*\"/', '/rel=\".*\"/'), array(WOWTOON_WOWHEAD_BASE_URL . "/?achievement=", '', ''), $txt);
      $newobj->description = $txt;
      $newobj->dateCompleted = $item->next()->innerHTML();
    }
    elseif (strpos($class, 'crit') !== FALSE) {
      $newobj->type = WOWTOON_FEED_CRITERIA;
      $newobj->icon = '';
      $txt = $item->firstChild()->firstChild()->innerHTML();
      $a = strpos($txt, 'Completed');
      $b = strpos($txt, '<div', $a);
      $txt = trim(substr($txt, $a, $b-$a));
      $txt = preg_replace(array("/achievement#.*:a/", '/onmouseover=\".*\"/', '/rel=\".*\"/'), array(WOWTOON_WOWHEAD_BASE_URL . "/?achievement=", '', ''), $txt);
      $newobj->description = $txt;//str_replace('"/wow/en/item/',WOWTOON_WOWHEAD_BASE_URL . '/' . WOWTOON_WOWHEAD_SEARCH_ID, $txt);
      $newobj->dateCompleted = $item->next()->innerHTML();
    }
    elseif (strpos($class, 'bosskill') !== FALSE) {
      $newobj->type = WOWTOON_FEED_BOSS_KILL;
      $newobj->icon = '';
      $txt = $item->firstChild()->firstChild()->innerHTML();
      $newobj->description = trim(strip_tags($txt));
      $newobj->dateCompleted = $item->next()->innerHTML();
    }
    else {
      $newobj->type = WOWTOON_FEED_OTHER;
      $txt = $item->firstChild()->firstChild()->firstChild()->firstChild()->attr('style');
      $a = strpos($txt, 'url("') + 5;
      $b = strpos($txt, '"', $a);
      $newobj->icon = substr($txt, $a, $b - $a);
      $txt = $item->parent()->parent()->innerHTML();
      $a = strpos($txt, '</a>') + 5;
      $txt = trim(substr($txt, $a));
      $newobj->description = str_replace('/wow/' . $language . '/item/', WOWTOON_WOWHEAD_BASE_URL . '/' . WOWTOON_WOWHEAD_SEARCH_ID, $txt);
      $newobj->dateCompleted = $item->next()->innerHTML();
    }
    
    // depending on the format of the country, we can't yet know what represent each $date
    //we'll have to specify in the switch, to get back to an iso date
    switch ($language) {
     case 'fr':
       if (strpos($newobj->dateCompleted, 'il y a') !== FALSE) {
         // replace 'il y a' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^il y a /', '/jour$/', '/jours$/', '/heure$/', '/heures$/', '/minute$/', '/minutes$/'),               
             array('', 'day ago', 'days ago', 'hour ago', 'hours ago', 'minute ago', 'minutes ago'),  
             $newobj->dateCompleted
           );
       }
       elseif (strpos($newobj->dateCompleted, 'dans') !== FALSE) {
         // replace 'dans' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^dans /', '/jour$/', '/jours$/', '/heure$/', '/heures$/', '/minute$/', '/minutes$/'),              
             array('+', 'day', 'days', 'hour', 'hours', 'minute', 'minutes'),    
             $newobj->dateCompleted
           );
       }
       else {
          // swap DD/MM/YYYY to MM/DD/YYYY and pass to strtotime().
         $tab = explode("/", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1] . '/' . $tab[0] . '/' . $tab[2]; 
       }
       break;
     case 'es':
       if (strpos($newobj->dateCompleted, 'hace') !== FALSE) {
         // replace 'il y a' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^hace /', '/día$/', '/días$/', '/hora$/', '/horas$/', '/minuto$/', '/minutos$/'),               
             array('', 'day ago', 'days ago', 'hour ago', 'hours ago', 'minute ago', 'minutes ago'),  
             $newobj->dateCompleted
           );
       }
       elseif (strpos($newobj->dateCompleted, 'en') !== FALSE) {
         // replace 'dans' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^en /', '/día$/', '/días$/', '/hora$/', '/horas$/', '/minuto$/', '/minutos$/'),               
             array('+', 'day', 'days', 'hour', 'hours', 'minute', 'minutes'),  
             $newobj->dateCompleted
           );
       }
       else {
         // swap DD/MM/YYYY to MM/DD/YYYY and pass to strtotime().
         $tab = explode("/", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1] . '/' . $tab[0] . '/' . $tab[2]; 
       }
       break;
     case 'de':
       if (strpos($newobj->dateCompleted, 'vor') !== FALSE) {
         // replace 'il y a' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace
           (
             array('/^vor /', '/Tag$/', '/Tagen$/', '/Stunde$/', '/Stunden$/', '/Minute$/', '/Minuten$/'),              
             array('', 'day ago', 'days ago', 'hour ago', 'hours ago', 'minute ago', 'minutes ago'),  
             $newobj->dateCompleted
           );
       }
       elseif (strpos($newobj->dateCompleted, 'in') !== FALSE) {
         // replace 'dans' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace
           (
             array('/^in /', '/Tag$/', '/Tagen$/', '/Stunde$/','/Stunden$/','/Minute$/', '/Minuten$/'),               
             array('+', 'day', 'days', 'hour', 'hours', 'minute', 'minutes'),  
             $newobj->dateCompleted
           );
       }
       else {
         // swap DD.MM.YYYY to MM/DD/YYYY and pass to strtotime().
         $tab = explode(".", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1] . '/' . $tab[0] . '/' . $tab[2]; 
       }
       break;
    case 'ru':
       if (strpos($newobj->dateCompleted, 'назад') !== FALSE) {
         // replace 'il y a' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/дн/', '/ч/', '/мин/', '/ назад$/'),              
             array('days ago', 'hours ago', 'minutes ago', ''),  
             $newobj->dateCompleted
           );
       }
       elseif (strpos($newobj->dateCompleted, 'через') !== FALSE) {
         // replace 'dans' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^через /', '/дн$/', '/дн.$/', '/ч$/', '/мин$/'),              
             array('+', 'day', 'days', 'hours', 'minutes'),  
             $newobj->dateCompleted
           );
       }
       else {
          // swap DD/MM/YYYY to MM/DD/YYYY and pass to strtotime().
         $tab = explode("/", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1]  . '/'.$tab[0] . '/' . $tab[2]; 
       }
       break;
    default:
    case 'en':
      if (strpos($newobj->dateCompleted, 'ago') !== FALSE) {
        // do nothing, string is in the proper format.
      }
      elseif (strpos($newobj->dateCompleted, 'in') !== FALSE) {
       // replace 'in' with '+'
       $newobj->dateCompleted = preg_replace(
           '/^in /',
           '+',
           $newobj->dateCompleted
         );
      }
      else {
        // us.battle.net uses MM/DD/YYYY, eu.battle.net uses DD/MM/YYYY
        if (strtolower($zone) == 'eu') {
         $tab = explode("/", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1] . '/' . $tab[0] . '/' . $tab[2]; 
        }
      }
      break;
    }
    $newobj->dateCompleted = strtotime($newobj->dateCompleted);    
    
    
    $ach[] = $newobj;
  }
  if ($ach) {
    db_delete('wowtoon_saved_achievements')
      ->condition('keyid', $toon->tid)
      ->condition('type', WOWTOON_FEED_GUILD_ACHIEVEMENT, '<')
      ->execute();
    foreach ($ach as $newobj) {
      drupal_write_record('wowtoon_saved_achievements', $newobj);
    }
    //drupal_set_message(t('Updated @name achievements', array('@name' => $name)));
  }
}

/**
 * Updates a guild from the armory, or checks for existance of guild.
 * 
 * @param string $realm
 *   - This can be the realm name, OR the {wowtoon}.tid to be updated.
 * @param string $name
 * @param integer $validate_slot
 *   - If this is passed see if the user has equiped / unequiped the item in the slot and return TRUE/FALSE. 
 * 
 * If {wowguild}.gid is passed, then we know it exists, so just update the guild.
 * 
 */
function wowguild_update_armory($realm = '', $name = '', $force = FALSE) {
  $guild = new stdClass();
  if (is_numeric($realm)) {
    // We are updating an existing guild.
    $guild = wowguild_load($realm);
    if ($guild === FALSE) {
      watchdog('error', 'Recevied an update for {wowguild}.gid = @gid, but it does not exist.', array('@gid' => $realm));
      return FALSE;
    }
  }
  else {
    // Run Error Checks
    $guild->guild_key = wowtoon_getkey($realm, $name);
    if (_wowtoon_validate_realm($realm) === FALSE) {
      return WOWTOON_UPDATE_INVALID_REALM;
    }
    if (trim($name) == '') {
      return WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
    }
    
    // See if guild exists and is owned.
    $result = db_query('SELECT gid FROM {wowguild_entity} WHERE guild_key = :guild_key', array(':guild_key' => $guild->guild_key))->fetchObject();
    if (!empty($result->gid)) {
      $guild = wowguild_load($result->gid);
    }
    else {
      $guild->realm = $realm;
      $guild->name = $name;
    }
  }
  
  $zone = substr($guild->realm, strlen($guild->realm) - 2);
  $realm = substr($guild->realm, 0, strlen($guild->realm) - 3);
  
   // wowhead uses www, the armory uses en
  $language = variable_get('wowtoon_default_language', 'www');
  if ($language == 'www') {
    $language = 'en';
  }
  // Verify that this zone is available to the selected language.
  if (strtolower($zone) == 'us' && ($language == 'fr' || $language == 'de' || $language == 'ru')) {
    $language = 'en';
  }
  
  $headers = array();
  $headers["User-Agent"] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/20070219 Firefox/2.0.0.2";
 // $url = sprintf('http://%s.battle.net/wow/en/guild/%s/%s/', strtolower($zone), urlencode(str_replace(array("'", ' '), array("", '-'), $realm)), str_replace(' ', '%20', $guild->name));
  $url = sprintf('http://%s.battle.net/wow/%s/guild/%s/%s/', strtolower($zone), $language,urlencode(str_replace(array("'", ' '), array("", '-'), $realm)), str_replace(' ', '%20', $guild->name));
  $guild->url = $url;
  $url .= "roster?view=guildActivity";
  
  if (module_exists('qpcache')) {
    $cache = qpcache_get($url);
    if (empty($cache)) {
      $result = drupal_http_request($url, $headers);
    }
    else {
      //drupal_set_message("Cache Hit");
      $result->data = $cache->xml;
    }
  }
  else {
    $result = drupal_http_request($url, $headers);
  }
  
  // Cache data if needed.
  if (empty($cache) && module_exists('qpcache')) {
    qpcache_set($url, $result->data, '+' . variable_get('wowtoon_updatedelta', 12) . ' hours'); 
  }
  
  
  try {
    $qp = qp($result->data, NULL, array('use_parser' => 'xml'));
    $error = $qp->find('#server-error')->innerHTML();
    if ($error) {
      if (strpos($error, 'Character Not Available')) {
        //drupal_set_message('Character not available on the Armory.');
        return WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
      }
      else { // if (strpos($error, 'Page Not Found'))
        //drupal_set_message('Error: Invalid Armory request.');
        return WOWTOON_UPDATE_INVALID_PAGE;
      }
    }
  }
  catch (Exception $e) {
    return WOWTOON_UPDATE_INVALID_PAGE;
  }

  $qp->top(); $guild->name = $qp->find('.profile-sidebar-tabard')->find('.name')->firstChild()->innerHTML();
  
  $qp->top(); $guild->faction = $qp->find('.profile-sidebar-tabard')->find('.faction')->innerHTML();
  //$qp->top(); $guild->members = $qp->find('.profile-sidebar-tabard')->find('.members')->innerHTML();
  $qp->top(); $guild->level = $qp->find('.profile-sidebar-tabard')->find('.level')->firstChild()->innerHTML();
  $qp->top(); $guild->battlegroup = $qp->find('.profile-sidebar-tabard')->find('.realm')->firstChild()->attr('data-battlegroup');
  
  if (variable_get('wowtoon_html5_tabards', 1)) {
    // Guild Tabard Settings: extract parameters from:
    // Blizzard file exists: http://[zone].battle.net/wow/static/images/guild/tabards
    // Local files: public://variable_get('wowtoon_html5_tabards_cache_path', 'tabardcache')/[zone]
    //
    // var tabard = new GuildTabard('guild-tabard', {
    //        'ring': 'alliance',
    //        'bg': [ 0, 44 ],
    //        'border': [ 0, 16 ],
    //        'emblem': [ 126, 16 ]
    //      });
    $a = strpos($result->data, 'new GuildTabard');
    $a = strpos($result->data, '{', $a);
    $b = strpos($result->data, '}', $a);
    $tabard = substr($result->data, $a, $b - $a + 1);
    $guild->tabard = json_decode(str_replace("'", '"', $tabard));
    // Build list of files needed to build tabard.  Make sure all filenames are safe by casting input varables into zero padded integers.
    $files = array(
      sprintf('shadow_%02d.png', $guild->tabard->bg[0]),
      sprintf('bg_%02d.png', $guild->tabard->bg[0]),
      sprintf('overlay_%02d.png', $guild->tabard->bg[0]),
      sprintf('border_%02d.png', max($guild->tabard->border[0], 0)),  // TODO: Fix this hack to allow -1???
      sprintf('emblem_%02d.png', $guild->tabard->emblem[0]),
      'hooks.png'
    );
    // Assert that $guild->tabard->ring is alliance|horde.  Just making sure since we are sending these filenames to the file system.
    if ($guild->tabard->ring == 'alliance') {
      $files[] = 'ring-alliance.png';
    }
    elseif ($guild->tabard->ring == 'horde') {
      $files[] = 'ring-horde.png';
    }
    
    
    $real_tabard_directory = variable_get('wowtoon_html5_tabards_cache_path', 'public://tabardcache');
    if (substr($real_tabard_directory, 0, 9) == 'public://') {
      $url_tabard_directory = file_create_url($real_tabard_directory);
      $real_tabard_directory = drupal_realpath($real_tabard_directory);
    }
    else {
      $url_tabard_directory = variable_get('wowtoon_html5_tabards_cache_url', file_create_url('public://tabardcache'));
    }
    if (!file_exists($url_tabard_directory)) {
      drupal_mkdir($url_tabard_directory, NULL, TRUE);
    }
    
    $zone = strtolower(substr($guild->realm, strlen($guild->realm) - 2));
    $real_tabard_directory .= '/' . $zone;
    $urlpath = 'http://' . $zone . '.battle.net/wow/static/images/guild/tabards';
    if (!file_exists($real_tabard_directory)) {
      drupal_mkdir($real_tabard_directory, NULL, TRUE);
    }
    foreach ($files as $file) {
      // Download file if needed.
      if (!file_exists($real_tabard_directory . '/' . $file)) {
        copy($urlpath . '/' . $file, $real_tabard_directory . '/' . $file);
      }
    }
  
  }
  $guild->updated = time();
  if (empty($guild->gid)) {
    $guild->added_to_database = REQUEST_TIME;
  }
  
  // Start Trasnaction
  $transaction = db_transaction();
  
  wowguild_save($guild);
  
  // Loop over memebers and create stats
  $members = array();
  $added = 0;
  $updated = 0;
  $current_page = 1;
  do {
    $headers = array();
    $headers["User-Agent"] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/20070219 Firefox/2.0.0.2";
    //$url = sprintf('http://%s.battle.net/wow/en/guild/%s/%s/', strtolower($zone), urlencode(str_replace(array("'", ' '), array("", '-'), $realm)), str_replace(' ', '%20', $guild->name));
    $url = sprintf('http://%s.battle.net/wow/%s/guild/%s/%s/', strtolower($zone), $language,urlencode(str_replace(array("'", ' '), array("", '-'), $realm)), str_replace(' ', '%20', $guild->name));
    $guild->url = $url;
    $url .= "roster?view=guildActivity&page=" . $current_page;
    //$result = drupal_http_request($url, $headers);
    
    if (module_exists('qpcache')) {
      $cache = qpcache_get($url);
      if (empty($cache)) {
        $result = drupal_http_request($url, $headers);
      }
      else {
        //drupal_set_message("Cache Hit");
        $result->data = $cache->xml;
      }
    }
    else {
      $result = drupal_http_request($url, $headers);
    }
    
    // Cache data if needed.
    if (empty($cache) && module_exists('qpcache')) {
      qpcache_set($url, $result->data, '+' . variable_get('wowtoon_updatedelta', 12) . ' hours'); 
    }
    
    
    
    $current_page++;
  
    try {
      $qp = qp($result->data, NULL, array('use_parser' => 'xml'));
      $error = $qp->find('#server-error')->innerHTML();
      if ($error) {
        if (strpos($error, 'Character Not Available')) {
          //drupal_set_message('Character not available on the Armory.');
          return WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
        }
        else { // if (strpos($error, 'Page Not Found'))
          //drupal_set_message('Error: Invalid Armory request.');
          return WOWTOON_UPDATE_INVALID_PAGE;
        }
      }
    }
    catch (Exception $e) {
      return WOWTOON_UPDATE_INVALID_PAGE;
    }  
    
    $qp->top(); $qp->find('#roster')->firstChild()->firstChild()->next()->children();
    $members_found_this_page_request = 0;
    foreach ($qp as $member) {
      unset($newtoon);
      // Get Data
      foreach ($member->children() as $col) {
        if ($col->hasClass('name')) {
          $newtoon['name'] = strip_tags($col->firstChild()->innerHTML());
        }
        if ($col->hasClass('race')) {
          $newtoon['race'] = $col->firstChild()->attr('data-tooltip');
          $img = array_pop(explode('/', $col->firstChild()->attr('src')));
          list($null, $newtoon['raceId'], $newtoon['genderId']) = explode('_', array_shift(explode('.', $img)));
        }
        if ($col->hasClass('cls')) {
          $newtoon['classId'] = $col->attr('data-raw');
          $newtoon['class'] = $col->firstChild()->attr('data-tooltip');
        }
        if ($col->hasClass('lvl')) {
          $newtoon['level'] = $col->innerHTML();
        }
        if ($col->hasClass('rank')) {
          $newtoon['rank'] = $col->attr('data-raw');
        }
        if ($col->hasClass('lifetime')) {
          $newtoon['lifetime'] = $col->innerHTML();
        }
        if ($col->hasClass('weekly')) {
          $newtoon['weekly'] = $col->innerHTML();
        }
      }
      // See if toon alrady exists.
      if (isset($newtoon['name'])) {
        $members_found_this_page_request++;
        $toon = wowtoon_load_by_realm_name($guild->realm, $newtoon['name']);
        $added++;
        if (!isset($toon->tid)) {
          // Creating Toon
          $toon->name = $newtoon['name'];
          $toon->realm = $guild->realm;
        }
        else {
          $updated++;
        }
        
        $toon->class = $newtoon['class'];
        $toon->classId = $newtoon['classId'];
        $toon->race = $newtoon['race'];
        $toon->raceId = $newtoon['raceId'];
        $toon->genderId = $newtoon['genderId'];
        $toon->battlegroup = $guild->battlegroup;
        $toon->level = $newtoon['level'];
        
        $toon->guild_rank = $newtoon['rank'];
        $toon->guild_name = $guild->name;
        $toon->gid = $guild->gid;
        $toon->guild_lifetime = !empty($newtoon['lifetime'])?$newtoon['lifetime']:0;
        $toon->guild_weekly = !empty($newtoon['weekly'])?$newtoon['weekly']:0;
        
        wowtoon_save($toon);
        $members[$toon->tid] = $toon->toon_key;
      }
    }
  } while ($current_page <= 10 && $members_found_this_page_request == 100); // This is a hard limit of 10 pages.  Only grab the next page if we found 100 members.
  
  
  // Remove all toons that are not on this list.
  $removed = db_update('wowtoon_entity')
    ->fields(array(
      'gid' => NULL
    ))
    ->condition('gid', $guild->gid)
    ->condition('tid', array_keys($members), 'NOT IN')
    ->execute();

  watchdog('wowtoon', 'Updated %name on %realm (%added added, %updated updated, %removed removed)',
    array('%name' => $guild->name, '%realm' => $guild->realm, '%added' => $added, '%updated' => $updated, '%removed' => $removed));
  return $guild;
}

/**
 * Updates the feed of a guild.
 * 
 * @param {wowguild_entity} $guild
 *   - This must be a valid guild entity.
 * 
 */
function wowguild_update_feed($guild) {
  wowguild_load_members($guild);
  $namelookup = array();
  foreach ($guild->members as $member) {
    $namelookup[$member->name] = $member;
  }
  
  // Generate conditional GET headers.
  $headers = array();
  $headers["User-Agent"] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/20070219 Firefox/2.0.0.2";
//  $headers['If-Modified-Since'] = gmdate('D, d M Y H:i:s', $feed['modified']) .' GMT';

  
  $zone = substr($guild->realm, strlen($guild->realm) - 2);
  $realm = substr($guild->realm, 0, strlen($guild->realm) - 3);
  
  // wowhead uses www, the armory uses en
  $language = variable_get('wowtoon_default_language', 'www');
  if ($language == 'www') {
    $language = 'en';
  }
  // Verify that this zone is available to the selected language.
  if (strtolower($zone) == 'us' && ($language == 'fr' || $language == 'de' || $language == 'ru')) {
    $language = 'en';
  }
  
  $url = $guild->url . 'news';

  if (module_exists('qpcache')) {
    $cache = qpcache_get($url);
    if (empty($cache)) {
      $result = drupal_http_request($url, $headers);
    }
    else {
      //drupal_set_message("Cache Hit");
      $result->data = $cache->xml;
    }
  }
  else {
    $result = drupal_http_request($url, $headers);
  }
  // Cache data if needed.
  if (empty($cache) && module_exists('qpcache')) {
    qpcache_set($url, $result->data, '+' . variable_get('wowtoon_updatedelta', 12) . ' hours'); 
  }
  
  
  try {
    $qp = qp($result->data, NULL, array('use_parser' => 'xml'));
    $error = $qp->find('#server-error')->innerHTML();
    if ($error) {
      if (strpos($error, 'Character Not Available')) {
        return WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
      }
      else {
        return WOWTOON_UPDATE_INVALID_PAGE;
      }
    }
  }
  catch (Exception $e) {
    return WOWTOON_UPDATE_INVALID_PAGE;
  }

  $ach = array();
  
  $qp->top()->find('.activity-feed')->find('a');
  foreach ($qp as $link) {
    if (strpos($link->attr('href'), 'achievement#') !== FALSE) {
      $a = strpos($link->attr('href'), ':a') + 2;
      $txt = trim(substr($link->attr('href'), $a));
      $link->attr('href', WOWTOON_WOWHEAD_BASE_URL . "/?achievement=" . $txt);
    }
    elseif (strpos($link->attr('href'), '/item/') !== FALSE) {
      $link->attr('href', WOWTOON_WOWHEAD_BASE_URL . "/?item=" . array_pop(explode('/', $link->attr('href'))));
    }
    elseif (strpos($link->attr('href'), '/character/') !== FALSE) {
      if (array_key_exists($link->innerHTML(), $namelookup)) {
        $toon = $namelookup[$link->innerHTML()];
        // If we are doing first update, we might not know classid.
        if (isset($toon->classid)) {
          $link->attr('class', 'shadow color-c' . $toon->classid);
        }
        $link->attr('href', url('toon/' . $toon->tid));
      }
    }
    $link->attr('onmouseover', '');
    $link->attr('rel', '');
  }
  
  $qp->top(); $qp->find('.activity-feed')->children();
  foreach ($qp as $item) {
    unset($newobj);
    $newobj->keyid = $guild->gid;
    $class = $item->attr('class');
    $newobj->class = $class;
    if (strpos($class, 'guild-levelup') !== FALSE) {
      $newobj->type = WOWTOON_FEED_GUILD_LEVEL_UP;
    }
    elseif (strpos($class, 'player-ach') !== FALSE) {
      $newobj->type = WOWTOON_FEED_GUILD_PLAYER_ACHIEVEMENT;
    }
    elseif (strpos($class, 'guild-ach') !== FALSE) {
      $newobj->type = WOWTOON_FEED_GUILD_ACHIEVEMENT;
    }
    elseif (strpos($class, 'item-crafted') !== FALSE) {
      $newobj->type = WOWTOON_FEED_GUILD_ITEM_CRAFTED;
    }
    elseif (strpos($class, 'item-purchased') !== FALSE) {
      $newobj->type = WOWTOON_FEED_GUILD_ITEM_PURCHASED;
    }
    elseif (strpos($class, 'item-looted') !== FALSE) {
      $newobj->type = WOWTOON_FEED_GUILD_ITEM_LOOTED;
    }
    
    switch ($newobj->type) {
      case WOWTOON_FEED_GUILD_LEVEL_UP:
        $newobj->icon = '';
        $txt = strip_tags($item->firstChild()->firstChild()->innerHTML());
        break;
      case WOWTOON_FEED_GUILD_PLAYER_ACHIEVEMENT:
      case WOWTOON_FEED_GUILD_ACHIEVEMENT:
        $txt = $item->firstChild()->firstChild()->firstChild()->firstChild()->attr('style');
        $a = strpos($txt, 'url("') + 5;
        $b = strpos($txt, '"', $a);
        $newobj->icon = substr($txt, $a, $b - $a);
        $txt = $item->parent()->parent()->innerHTML();
        $a = strpos($txt, '</a>') + 5;
        $txt = trim(substr($txt, $a));
        break;
      default:
        $txt = $item->firstChild()->firstChild()->firstChild()->firstChild()->attr('style');
        $a = strpos($txt, 'url("') + 5;
        $b = strpos($txt, '"', $a);
        $newobj->icon = substr($txt, $a, $b - $a);
        
        $txt = $item->parent()->parent()->innerHTML();
        $a = strpos($txt, '</a>') + 5;
        $txt = trim(substr($txt, $a));
    }
    /*
    if (!isset($newobj->description)) {
      $tmp = qp($txt);
      $tmp->top()->find('p');
      if ($tmp->html()) {
        $newobj->description = trim($tmp->innerHTML());
      } else {
        $tmp->top()->find('body');
        $newobj->description = trim($tmp->innerHTML());
      }
    }*/
    $newobj->description = $txt;
    
    $newobj->dateCompleted = $item->next()->innerHTML();
    
     // depending on the format of the country, we can't yet know what represent each $date
    //we'll have to specify in the switch, to get back to an iso date
    switch ($language) {
     case 'fr':
       if (strpos($newobj->dateCompleted, 'il y a') !== FALSE) {
         // replace 'il y a' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^il y a /', '/jour$/', '/jours$/', '/heure$/', '/heures$/', '/minute$/', '/minutes$/'),               
             array('', 'day ago', 'days ago', 'hour ago', 'hours ago', 'minute ago', 'minutes ago'),  
             $newobj->dateCompleted
           );
       }
       elseif (strpos($newobj->dateCompleted, 'dans') !== FALSE) {
         // replace 'dans' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^dans /', '/jour$/', '/jours$/', '/heure$/', '/heures$/', '/minute$/', '/minutes$/'),              
             array('+', 'day', 'days', 'hour', 'hours', 'minute', 'minutes'),    
             $newobj->dateCompleted
           );
       }
       else {
          // swap DD/MM/YYYY to MM/DD/YYYY and pass to strtotime().
         $tab = explode("/", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1] . '/' . $tab[0] . '/' . $tab[2]; 
       }
       break;
     case 'es':
       if (strpos($newobj->dateCompleted, 'hace') !== FALSE) {
         // replace 'il y a' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^hace /', '/día$/', '/días$/', '/hora$/', '/horas$/', '/minuto$/', '/minutos$/'),               
             array('', 'day ago', 'days ago', 'hour ago', 'hours ago', 'minute ago', 'minutes ago'),  
             $newobj->dateCompleted
           );
       }
       elseif (strpos($newobj->dateCompleted, 'en') !== FALSE) {
         // replace 'dans' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^en /', '/día$/', '/días$/', '/hora$/', '/horas$/', '/minuto$/', '/minutos$/'),               
             array('+', 'day', 'days', 'hour', 'hours', 'minute', 'minutes'),  
             $newobj->dateCompleted
           );
       }
       else {
         // swap DD/MM/YYYY to MM/DD/YYYY and pass to strtotime().
         $tab = explode("/", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1] . '/' . $tab[0] . '/' . $tab[2]; 
       }
       break;
     case 'de':
       if (strpos($newobj->dateCompleted, 'vor') !== FALSE) {
         // replace 'il y a' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace
           (
             array('/^vor /', '/Tag$/', '/Tagen$/', '/Stunde$/', '/Stunden$/', '/Minute$/', '/Minuten$/'),              
             array('', 'day ago', 'days ago', 'hour ago', 'hours ago', 'minute ago', 'minutes ago'),  
             $newobj->dateCompleted
           );
       }
       elseif (strpos($newobj->dateCompleted, 'in') !== FALSE) {
         // replace 'dans' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace
           (
             array('/^in /', '/Tag$/', '/Tagen$/', '/Stunde$/','/Stunden$/','/Minute$/', '/Minuten$/'),               
             array('+', 'day', 'days', 'hour', 'hours', 'minute', 'minutes'),  
             $newobj->dateCompleted
           );
       }
       else {
         // swap DD.MM.YYYY to MM/DD/YYYY and pass to strtotime().
         $tab = explode(".", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1] . '/' . $tab[0] . '/' . $tab[2]; 
       }
       break;
    case 'ru':
       if (strpos($newobj->dateCompleted, 'назад') !== FALSE) {
         // replace 'il y a' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/дн/', '/ч/', '/мин/', '/ назад$/'),              
             array('days ago', 'hours ago', 'minutes ago', ''),  
             $newobj->dateCompleted
           );
       }
       elseif (strpos($newobj->dateCompleted, 'через') !== FALSE) {
         // replace 'dans' with '',and jours with days ago, heures with hours ago...  etc.
         $newobj->dateCompleted = preg_replace(
             array('/^через /', '/дн$/', '/дн.$/', '/ч$/', '/мин$/'),              
             array('+', 'day', 'days', 'hours', 'minutes'),  
             $newobj->dateCompleted
           );
       }
       else {
          // swap DD/MM/YYYY to MM/DD/YYYY and pass to strtotime().
         $tab = explode("/", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1]  . '/'.$tab[0] . '/' . $tab[2]; 
       }
       break;
    default:
    case 'en':
      if (strpos($newobj->dateCompleted, 'ago') !== FALSE) {
        // do nothing, string is in the proper format.
      }
      elseif (strpos($newobj->dateCompleted, 'in') !== FALSE) {
       // replace 'in' with '+'
       $newobj->dateCompleted = preg_replace(
           '/^in /',
           '+',
           $newobj->dateCompleted
         );
      }
      else {
        // us.battle.net uses MM/DD/YYYY, eu.battle.net uses DD/MM/YYYY
        if (strtolower($zone) == 'eu') {
         $tab = explode("/", $newobj->dateCompleted);
         $newobj->dateCompleted = $tab[1] . '/' . $tab[0] . '/' . $tab[2]; 
        }
      }
      break;
    }
    
    if (strpos($newobj->dateCompleted, 'ago') !== FALSE) {
      $newobj->dateCompleted = strtotime($newobj->dateCompleted, REQUEST_TIME);
    }
    else {
      $newobj->dateCompleted = strtotime($newobj->dateCompleted);
    }
    $ach[] = $newobj;
  }
  
  if ($ach) {
    db_delete('wowtoon_saved_achievements')
      ->condition('keyid', $guild->gid)
      ->condition('type', WOWTOON_FEED_GUILD_ACHIEVEMENT, '>=')
      ->execute();
    foreach ($ach as $newobj) {
      drupal_write_record('wowtoon_saved_achievements', $newobj);
    }
  }
}
/**
 * Updates the progression of a guild.
 * 
 * @param {wowguild_entity} $guild
 *   - This must be a valid guild entity.
 * 
 */
function wowguild_update_progression($guild) {
  // Generate conditional GET headers.
  $headers = array();
  $headers["User-Agent"] = "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2) Gecko/20070219 Firefox/2.0.0.2";
//  $headers['If-Modified-Since'] = gmdate('D, d M Y H:i:s', $feed['modified']) .' GMT';

  //http://us.battle.net/wow/en/guild/senjin/premonition/achievement/15087
  $url = $guild->url . 'achievement/' . WOWTOON_RAID_PROGRESSION_ACHIEVEMENT_ID;

  if (module_exists('qpcache')) {
    $cache = qpcache_get($url);
    if (empty($cache)) {
      $result = drupal_http_request($url, $headers);
    }
    else {
      //drupal_set_message("Cache Hit");
      $result->data = $cache->xml;
    }
  }
  else {
    $result = drupal_http_request($url, $headers);
  }

  // Cache data if needed.
  if (empty($cache) && module_exists('qpcache')) {
    qpcache_set($url, $result->data, '+' . variable_get('wowtoon_updatedelta', 12) . ' hours'); 
  }
  
    
  try {
    $qp = qp($result->data, NULL, array('use_parser' => 'xml'));
    $error = $qp->find('#server-error')->innerHTML();
    if ($error) {
      if (strpos($error, 'Character Not Available')) {
        return WOWTOON_UPDATE_CHARACTER_NOT_AVAILABLE;
      }
      else {
        return WOWTOON_UPDATE_INVALID_PAGE;
      }
    }
  }
  catch (Exception $e) {
    return WOWTOON_UPDATE_INVALID_PAGE;
  }

  $encounters = array();
  
  $qp->top()->find('ul')->children();
  foreach ($qp as $link) {
    $ach_id = $link->attr('data-id');
    if ($ach_id) {
      $encounters[$ach_id] = array('ach_id' => $ach_id);
      if ($link->hasClass('locked')) {
        $encounters[$ach_id]['completed'] = 0;
      }
      else {
        $encounters[$ach_id]['completed'] = 1;
      }
      $txt = $link->firstChild()->innerHTML();
      list($null, $title, $description) = explode("\n", $txt);
      $encounters[$ach_id]['title'] = trim(strip_tags($title));
      $encounters[$ach_id]['description'] = trim(strip_tags($description));
      $encounters[$ach_id]['bosses'] = array();
      $kills = 0;
      $count = 0;
      $link->parent()->find('.meta-achievements')->firstChild()->children();
      foreach ($link as $meta) {
        $encounters[$ach_id]['bosses'][$count] = array('name' => trim($meta->innerHTML()));
        if ($meta->hasClass('unlocked')) {
          $kills++;
          $encounters[$ach_id]['bosses'][$count]['killed'] = 1;
        }
        else {
          $encounters[$ach_id]['bosses'][$count]['killed'] = 0;
        }
        $count++;
      }
      if ($encounters[$ach_id]['completed'] && isset($encounters[$ach_id]['total']) && $encounters[$ach_id]['total'] == 1) {
        $encounters[$ach_id]['kills'] = 1;
        $encounters[$ach_id]['total'] = 1;
      }
      else {
        $encounters[$ach_id]['kills'] = $kills;
        $encounters[$ach_id]['total'] = $count;
      }
    }
  }
  if ($encounters) {
    db_update('wowguild_entity')
      ->fields(array('raid_progression' => serialize($encounters)))
      ->condition('gid', $guild->gid)
      ->execute();
  }
}
/**
 * Helper function that modifies the json input from blizzard to parse properly.
 * 
 * @param string $txt
 * @param integer $start
 * @param integer $end
 */
function _wowtoon_parse_json($txt, $start, $end) {
  $search = array("/\\", "\\'", 'nTrivialRaids:', 'nOptimalRaids:', 'nChallengingRaids:', 'name:', 'nKills:', 'optional:');
  $replace = array("/", "'", '"nTrivialRaids":', '"nOptimalRaids":', '"nChallengingRaids":', '"name":', '"nKills":', '"optional":');
  $a = strpos($txt, $start) + strlen($start);
  $b = strpos($txt, $end, $a) + 1;
  $c = '[' . preg_replace(array('/\t(\w*):/'), array('"${1}":'), substr($txt, $a, $b - $a)) . ']';
  $c = str_replace($search, $replace, $c);
  return json_decode($c);
}

